# require-yml

[![NPM Version][npm-image]][npm-url]
[![NPM Downloads][downloads-image]][downloads-url]
[![Build Status][travis-image]][travis-url]
[![Test Coverage][coveralls-image]][coveralls-url]

[English Doc](./README.md)

## 为什么造了个轮子

这个包是为了代替[require-yaml](https://www.npmjs.com/package/require-yaml)，因为[require.extensions已不被推荐使用了](http://nodejs.org/api/globals.html#globals_require_extensions)。

而且提供可以加载更多的文件格式（yml/yaml/json）及文件夹、支持每个配置文件迭代、支持异步加载。

## 安装

```bash
npm install require-yml
```

## 使用

配置文件目录结构:

```sh
configs/
  |- foo/
    |- bar/
      |- a.yml
      |- b.yaml
      |- c.json
    |- empty/
```

```javascript
var req = require('require-yml')
```

### 加载单个文件（yml/yaml/json）

```javascript
var yml = req('./configs/foo/bar/a.yml')
var yaml = req('./configs/foo/bar/b') // b.yaml
var json = req('./configs/foo/bar/c.json')
console.log(yml, yaml, json)
// >> {}, {}, {}
```

### 加载整个目录

```javascript
var all = req('./configs')
console.log(all)
// >> json object {"foo":{"bar":[Object Object]}
```

### 加载空文件/文件夹

```javascript
var empty = req('./configs/empty')
console.log(empty)
// >> undefined
```

### 迭代每个文件

```javascript
var iterator = function(json) {
  json.inject = 'everywhere'
  return json
}
var yml = req('./configs', iterator)
console.log(yml.foo.bar.a.inject)
// >> 'everywhere'
```

### 异常处理

```javascript
var yml = req('./configs', function brokenIterator(json) { 
   a = b // -> 抛出 `a is undefined`
})
req.onLoadError = function(err) {
  // 异常处理
  switch(e.CODE) {
    ...
  }
}
```

### 异步加载

```javascript
req('./configs', null, function(yml){
  console.log(yml.foo.bar.a)
})
// >> {}
```

## 测试

```sh
npm test
```

## License

[MIT](LICENSE)

[npm-image]: https://img.shields.io/npm/v/require-yml.svg?style=flat
[npm-url]: https://npmjs.org/package/require-yml
[travis-image]: https://travis-ci.org/cutsin/require-yml.svg
[travis-url]: https://travis-ci.org/cutsin/require-yml
[downloads-image]: https://img.shields.io/npm/dm/require-yml.svg?style=flat
[downloads-url]: https://npmjs.org/package/require-yml
[coveralls-image]: https://img.shields.io/coveralls/cutsin/require-yml.svg?style=flat
[coveralls-url]: https://coveralls.io/r/cutsin/require-yml